/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.attachments;

import de.ozgcloud.antragsraum.common.TechnicalException;
import io.grpc.stub.CallStreamObserver;
import lombok.AllArgsConstructor;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.io.IOUtils;

import java.io.IOException;
import java.io.InputStream;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.function.Function;

@Log4j2
@AllArgsConstructor
class ChunkedFileSender<T> {
	private final AtomicBoolean hasUploadFile = new AtomicBoolean(true);
	private final InputStream uploadStream;
	private final int chunkSize;
	private final Function<byte[], T> buildChunkRequest;
	private T requestMetadata;

	public void sendChunkTo(CallStreamObserver<T> streamObserver) {
		if (hasUploadFile.get()) {
			sendMetadata(streamObserver);
			int size = sendNextChunk(streamObserver);
			if (size < chunkSize) {
				handleFileEndReached(streamObserver);
			}
		}
	}

	private void sendMetadata(CallStreamObserver<T> streamObserver) {
		if (requestMetadata != null) {
			streamObserver.onNext(requestMetadata);
			requestMetadata = null;
		}
	}

	private int sendNextChunk(CallStreamObserver<T> streamObserver) {
		byte[] content = readFromStream();
		var size = content.length;
		if (size > 0) {
			streamObserver.onNext(buildChunkRequest.apply(content));
		}
		return size;
	}

	private byte[] readFromStream() {
		try {
			return uploadStream.readNBytes(chunkSize);
		} catch (IOException e) {
			LOG.error("Error on sending a single chunk", e);
			throw new TechnicalException(e);
		}
	}

	private void handleFileEndReached(CallStreamObserver<T> streamObserver) {
		IOUtils.closeQuietly(uploadStream);
		streamObserver.onCompleted();
		hasUploadFile.getAndSet(false);
	}
}
