/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.attachments;

import java.io.OutputStream;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

import org.springframework.stereotype.Component;

import com.google.protobuf.ByteString;

import de.ozgcloud.antragsraum.callcontext.ContextService;
import de.ozgcloud.antragsraum.common.ChannelPropertiesFactory;
import de.ozgcloud.antragsraum.common.TechnicalException;
import de.ozgcloud.vorgang.grpc.binaryFile.BinaryFileServiceGrpc;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcBinaryFilesRequest;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcFindFilesResponse;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcGetBinaryFileDataRequest;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcUploadBinaryFileMetaData;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcUploadBinaryFileRequest;
import io.grpc.stub.StreamObserver;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import net.devh.boot.grpc.client.channelfactory.GrpcChannelFactory;
import net.devh.boot.grpc.client.interceptor.GlobalClientInterceptorRegistry;

@Component
@RequiredArgsConstructor
class FileGrpcClient {
	public static final int CHUNK_SIZE = 255 * 1024;

	private final @NonNull GlobalClientInterceptorRegistry registry;
	private final @NonNull ChannelPropertiesFactory channelPropertiesFactory;
	private final @NonNull ContextService contextService;

	GrpcChannelFactory getChannelFactory(String channelAddress) {
		return de.ozgcloud.antragsraum.common.GrpcChannelFactory.getChannelFactory(channelPropertiesFactory.getGrpcChannelsProperties(channelAddress),
		  registry);
	}

	GrpcFindFilesResponse findBinaryFilesMetaData(GrpcBinaryFilesRequest request, String address) {
		try (GrpcChannelFactory channelFactory = getChannelFactory(address)) {
			var channel = channelFactory.createChannel(address);
			var serviceStub = BinaryFileServiceGrpc.newBlockingStub(channel);

			return getBinaryFilesMetaData(request, serviceStub);
		}
	}

	GrpcFindFilesResponse getBinaryFilesMetaData(GrpcBinaryFilesRequest request,
	  BinaryFileServiceGrpc.BinaryFileServiceBlockingStub fileServiceBlockingStub) {
		return fileServiceBlockingStub.findBinaryFilesMetaData(request);
	}

	StreamObserver<GrpcUploadBinaryFileRequest> uploadFile(OzgUploadFile uploadFile, String address, CompletableFuture<String> fileIdFuture) {
		try (GrpcChannelFactory channelFactory = getChannelFactory(address)) {
			var channel = channelFactory.createChannel(address);
			var responseObserver = createUploadFileObserver(fileIdFuture, uploadFile);
			var serviceStub = BinaryFileServiceGrpc.newStub(channel);

			return doUploadFiles(responseObserver, serviceStub);
		}
	}

	StreamObserver<GrpcUploadBinaryFileRequest> doUploadFiles(FileUploadStreamObserver observer,
	  BinaryFileServiceGrpc.BinaryFileServiceStub asyncServiceStub) {
		return asyncServiceStub.uploadBinaryFileAsStream(observer);
	}

	FileUploadStreamObserver createUploadFileObserver(CompletableFuture<String> fileIdFuture, OzgUploadFile uploadFile) {
		var metadataRequest = buildMetaDataRequest(uploadFile);
		var streamer = new ChunkedFileSender<>(uploadFile.fileContent(), CHUNK_SIZE, this::buildChunkRequest, metadataRequest);
		return new FileUploadStreamObserver(fileIdFuture, streamer);
	}

	private GrpcUploadBinaryFileRequest buildMetaDataRequest(OzgUploadFile uploadFile) {
		return GrpcUploadBinaryFileRequest.newBuilder()
		  .setMetadata(GrpcUploadBinaryFileMetaData.newBuilder()
			.setContext(contextService.createCallContext())
			.setVorgangId(uploadFile.fileData().getVorgangId())
			.setContentType(uploadFile.fileData().getContentType())
			.setFileName(uploadFile.fileData().getFileName()))
		  .build();
	}

	GrpcUploadBinaryFileRequest buildChunkRequest(byte[] bytes) {
		return GrpcUploadBinaryFileRequest.newBuilder().setFileContent((ByteString.copyFrom(bytes))).build();
	}

	void downloadFileContent(String fileId, OutputStream out, String address) {
		try (GrpcChannelFactory channelFactory = getChannelFactory(address)) {
			var channel = channelFactory.createChannel(address);
			var serviceStub = BinaryFileServiceGrpc.newStub(channel);

			doDownloadFileContent(fileId, out, serviceStub);
		}
	}

	void doDownloadFileContent(String fileId, OutputStream out, BinaryFileServiceGrpc.BinaryFileServiceStub binaryFileServiceStub) {
		var streamFuture = new CompletableFuture<Boolean>();
		var responseObserver = createDownloadFileObserver(streamFuture, out);
		binaryFileServiceStub.getBinaryFileContent(buildGrpcGetBinaryFileDataRequest(fileId), responseObserver);

		waitUntilFutureToComplete(streamFuture);

	}

	FileDownloadStreamObserver createDownloadFileObserver(CompletableFuture<Boolean> streamFuture, OutputStream out) {
		return new FileDownloadStreamObserver(streamFuture, out);
	}

	private GrpcGetBinaryFileDataRequest buildGrpcGetBinaryFileDataRequest(String fileId) {
		return GrpcGetBinaryFileDataRequest.newBuilder().setFileId(fileId).build();
	}

	void waitUntilFutureToComplete(CompletableFuture<Boolean> streamFuture) {
		try {
			streamFuture.get(10, TimeUnit.MINUTES);
		} catch (InterruptedException e) {
			Thread.currentThread().interrupt();
			throw new TechnicalException(e);
		} catch (ExecutionException | TimeoutException e) {
			throw new TechnicalException(e);
		}
	}
}
