/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.mocks;

import de.ozgcloud.antragsraum.common.TechnicalException;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcUploadBinaryFileRequest;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcUploadBinaryFileResponse;
import io.grpc.stub.StreamObserver;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

@Log4j2
public class UploadStreamObserverStub implements StreamObserver<GrpcUploadBinaryFileRequest> {
	private final StreamObserver<GrpcUploadBinaryFileResponse> responseObserver;
	private final StubDataRepository stubDataRepository;
	private ByteArrayOutputStream byteArrayOutput;
	private CompletableFuture<String> fileIdFuture;
	private String fileId;
	private String fileName;

	UploadStreamObserverStub(StreamObserver<GrpcUploadBinaryFileResponse> responseObserver, StubDataRepository stubDataRepository) {
		this.responseObserver = responseObserver;
		this.stubDataRepository = stubDataRepository;
	}

	@Override
	public void onNext(GrpcUploadBinaryFileRequest fileUploadRequest) {
		if (fileUploadRequest.hasMetadata()) {
			LOG.info("Received file metadata: " + fileUploadRequest.getMetadata());
			fileName = fileUploadRequest.getMetadata().getFileName();
			fileId = stubDataRepository.addFile(fileUploadRequest.getMetadata());
			fileIdFuture = CompletableFuture.completedFuture(fileId);
			byteArrayOutput = new ByteArrayOutputStream();
		} else {
			storeFileContent(fileUploadRequest);
		}
	}

	void storeFileContent(GrpcUploadBinaryFileRequest fileUploadRequest) {
		try {
			IOUtils.write(fileUploadRequest.getFileContent().toByteArray(), byteArrayOutput);
		} catch (IOException e) {
			throw new TechnicalException(e);
		}
	}

	@Override
	public void onError(Throwable t) {
		LOG.error("GrpcLargeFileRequestObserver onError has been called", t);
		throw new TechnicalException(t);
	}

	@Override
	public void onCompleted() {
		try {
			var fileContent = byteArrayOutput.toByteArray();
			stubDataRepository.addFileContent(fileId, fileContent);
			var file = File.createTempFile("upload-", fileName);
			FileUtils.writeByteArrayToFile(file, fileContent);
			LOG.info("Saved file {}", file.getAbsoluteFile());
			responseObserver.onNext(GrpcUploadBinaryFileResponse.newBuilder().setFileId(fileIdFuture.get(3, TimeUnit.SECONDS)).build());

			byteArrayOutput.close();
		} catch (ExecutionException | IOException | TimeoutException e) {
			handleException(e);
		} catch (InterruptedException ie) {
			handleException(ie);
			Thread.currentThread().interrupt();
		}
		responseObserver.onCompleted();

	}

	void handleException(Exception e) {
		responseObserver.onNext(GrpcUploadBinaryFileResponse.newBuilder().build());
		LOG.error(e.getMessage(), e);
		throw new TechnicalException(e);
	}
}
