/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.nachricht;

import java.time.DateTimeException;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.Comparator;
import java.util.List;
import java.util.Objects;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import de.ozgcloud.antragsraum.attachments.FileService;
import de.ozgcloud.antragsraum.attachments.OzgFile;
import de.ozgcloud.nachrichten.antragraum.GrpcRueckfrage;
import de.ozgcloud.nachrichten.antragraum.GrpcRueckfrageAnswer;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Service
@RequiredArgsConstructor
class NachrichtMapper {
	private final @NonNull FileService fileService;
	private final String EXPECTED_DATE_FORMAT = "yyyy-MM-ddTHH:mm:ss";

	Nachricht fromGrpcRueckfrage(GrpcRueckfrage grpcRueckfrage, String nachrichtEventId) {
		LOG.info("Mapping GrpcRueckfrage [{}] for NachrichtEvent [{}] to Nachricht", grpcRueckfrage, nachrichtEventId);
		var builder = Nachricht.builder()
		  .vorgangId(grpcRueckfrage.getVorgangId())
		  .id(grpcRueckfrage.getId())
		  .nachrichtEventId(nachrichtEventId)
		  .title(grpcRueckfrage.getVorgangName())
		  .message(grpcRueckfrage.getText())
		  .replyNachrichten(fromGrpcRueckfrageAnswer(grpcRueckfrage, nachrichtEventId))
		  .answeredAt(getAnsweredAt(grpcRueckfrage))
		  .sendAt(toMillisecondsTimestamp(grpcRueckfrage.getSentAt()))
		  .attachments(getAttachments(grpcRueckfrage.getAttachmentFileIdList(), nachrichtEventId))
		  .accessible(grpcRueckfrage.getAccessible())
		  .trustLevel(TrustLevelMapper.map(grpcRueckfrage.getTrustLevel()))
		  .status(grpcRueckfrage.getStatus());

		return builder.build();
	}

	// TODO: Entfernen, sobald valides answeredAt Datum gesendet wird
	long getAnsweredAt(GrpcRueckfrage rueckfrage) {
		var answeredAt = rueckfrage.getAnsweredAt();
		if (StringUtils.isEmpty(answeredAt) && CollectionUtils.isNotEmpty(rueckfrage.getAnswersList())) {
			LOG.info("Rückfrage answeredAt is not set, loading it from sendAt of the replies");
			var sortedBySendBy = rueckfrage.getAnswersList().stream().filter(Objects::nonNull)
			  .filter(answer -> StringUtils.isNotEmpty(answer.getSentAt()))
			  .sorted(
				Comparator.comparing(GrpcRueckfrageAnswer::getSentAt)).toList();

			if (sortedBySendBy.isEmpty()) {
				answeredAt = "";
			} else {
				answeredAt = sortedBySendBy.getLast().getSentAt().substring(0, EXPECTED_DATE_FORMAT.length());
			}
		}

		return toMillisecondsTimestamp(answeredAt);
	}

	private List<OzgFile> getAttachments(List<String> fileIds, String nachrichtEventId) {
		return fileIds.stream().filter(StringUtils::isNotEmpty).map(fileId -> fileService.getFile(fileId, nachrichtEventId))
		  .filter(Objects::nonNull).toList();
	}

	private List<ReplyNachricht> fromGrpcRueckfrageAnswer(GrpcRueckfrage rueckfrage, String nachrichtEventId) {
		return rueckfrage.getAnswersList().stream()
		  .map(answer -> {
			  var builder = ReplyNachricht.builder()
				.id(rueckfrage.getId())
				.nachrichtEventId(nachrichtEventId)
				.message(answer.getAnswerText())
				.attachments(getAttachments(answer.getAttachmentFileIdList(), nachrichtEventId));
			  return builder.build();
		  }).toList();
	}

	long toMillisecondsTimestamp(String dateString) {
		long timestamp = 0;
		try {
			timestamp = LocalDateTime.parse(dateString).toEpochSecond(ZoneOffset.UTC) * 1000;
		} catch (DateTimeException e) {
			LOG.warn("Invalid date [{}] received for Nachricht. Setting it to 0 (=1970-01-01)", dateString);
		}

		return timestamp;
	}

	GrpcRueckfrageAnswer toGrpcRueckfrageAnswer(ReplyNachricht nachricht) {
		var builder = GrpcRueckfrageAnswer.newBuilder()
		  .setRueckfrageId(nachricht.id())
		  .setAnswerText(nachricht.message());

		if (Objects.nonNull(nachricht.attachments())) {
			builder.addAllAttachmentFileId(nachricht.attachments().stream()
			  .map(OzgFile::id).toList());
		}

		return builder.build();
	}
}
