/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.nachricht;

import java.util.List;
import java.util.Objects;
import java.util.Optional;

import org.springframework.stereotype.Service;

import de.ozgcloud.antragsraum.callcontext.ContextService;
import de.ozgcloud.antragsraum.command.CommandReference;
import de.ozgcloud.antragsraum.events.NachrichtEvent;
import de.ozgcloud.nachrichten.antragraum.GrpcFindRueckfragenRequest;
import de.ozgcloud.nachrichten.antragraum.GrpcGetRueckfrageRequest;
import de.ozgcloud.nachrichten.antragraum.GrpcSendRueckfrageAnswerRequest;
import de.ozgcloud.nachrichten.antragraum.GrpcSendRueckfrageAnswerResponse;
import io.grpc.StatusRuntimeException;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Service
@Log4j2
@RequiredArgsConstructor
class NachrichtenRemoteService {
	private final @NonNull NachrichtenGrpcClient nachrichtenGrpcClient;
	private final @NonNull ContextService contextService;
	private final @NonNull NachrichtMapper nachrichtMapper;

	List<NachrichtHeader> findRueckfrageHeads(NachrichtEvent event) {
		try {
			var rfBuilder = GrpcFindRueckfragenRequest.newBuilder().setPostfachId(event.postfachId());
			if (Objects.nonNull(contextService.getSamlResponse())) {
				rfBuilder.setSamlToken(contextService.getSamlResponse());
			}

			var reply = nachrichtenGrpcClient.getFindRueckfragen(rfBuilder.build(), event.address());
			var rueckfrageHeads = reply.getRueckfrageHeadList();
			LOG.info("{} RueckfrageHeads für Postfach ID {} erhalten", rueckfrageHeads.size(), event.postfachId());

			return rueckfrageHeads.stream().map(rueckfrageHead -> NachrichtHeaderMapper.fromGrpcRueckfrageHead(rueckfrageHead, event.id())).toList();
		} catch (StatusRuntimeException e) {
			LOG.error("Error beim Laden der RueckfrageHeads für das Postfach {}", event.postfachId(), e);
		}

		return List.of();
	}

	Optional<Nachricht> getRueckfrage(String rueckfrageId, NachrichtEvent event) {
		try {
			var rfBuilder = GrpcGetRueckfrageRequest.newBuilder().setId(rueckfrageId);
			if (Objects.nonNull(contextService.getSamlResponse())) {
				rfBuilder.setSamlToken(contextService.getSamlResponse());
			}

			var reply = nachrichtenGrpcClient.getGetRueckfrage(rfBuilder.build(), event.address());
			var rueckfrage = reply.getRueckfrage();
			LOG.info("Rueckfrage [{}] für ID {} erhalten", rueckfrage, rueckfrageId);

			return Optional.of(nachrichtMapper.fromGrpcRueckfrage(reply.getRueckfrage(), event.id()));
		} catch (StatusRuntimeException e) {
			LOG.error("Error beim Laden der Rueckfrage mit ID {}", rueckfrageId, e);
		}

		return Optional.empty();
	}

	CommandReference sendAnswer(ReplyNachricht nachricht, String address) {
		var request = GrpcSendRueckfrageAnswerRequest.newBuilder()
		  .setAnswer(nachrichtMapper.toGrpcRueckfrageAnswer(nachricht))
		  .setSamlToken(contextService.getSamlResponse())
		  .build();

		GrpcSendRueckfrageAnswerResponse response = nachrichtenGrpcClient.answerRueckfrage(request, address);

		return CommandReference.builder()
		  .id(response.getCommandId())
		  .nachrichtEventId(nachricht.nachrichtEventId())
		  .build();
	}
}
