/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.attachments;

import static de.ozgcloud.antragsraum.attachments.FileController.*;
import static de.ozgcloud.antragsraum.attachments.OzgFileTestFactory.*;
import static de.ozgcloud.antragsraum.attachments.FileValidatorTest.*;
import static org.hamcrest.Matchers.*;
import static org.mockito.Mockito.any;
import static org.mockito.Mockito.*;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.request;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.util.concurrent.CompletableFuture;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.boot.test.mock.mockito.SpyBean;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.context.junit.jupiter.SpringJUnitConfig;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.web.client.RestTemplate;

import de.ozgcloud.antragsraum.common.NotFoundException;
import de.ozgcloud.antragsraum.common.TechnicalException;
import de.ozgcloud.antragsraum.common.VirusFoundException;
import de.ozgcloud.antragsraum.nachricht.NachrichtTestFactory;

@AutoConfigureMockMvc
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.MOCK)
@SpringJUnitConfig(classes = { FileControllerTestConfiguration.class })
public class FileControllerITCase {
	@SpyBean
	private FileController controller;

	@MockBean
	private FileService fileService;

	@MockBean
	private RestTemplate restTemplate;

	@Autowired
	private MockMvc mockMvc;

	@Nested
	class TestDownloadingFileContent {
		@BeforeEach
		void init() {
			when(fileService.getFile(anyString(), anyString())).thenReturn(OzgFileTestFactory.create());
		}

		@Test
		@WithMockUser
		void shouldGetFileContent() throws Exception {
			performRequest().andExpect(status().isOk());
		}
	}

	@Nested
	class TestFileNotFoundError {
		@BeforeEach
		void init() {
			doThrow(NotFoundException.class).when(fileService).getFile(anyString(), anyString());
		}

		@Test
		@WithMockUser
		void shouldReturn404() throws Exception {
			performRequest().andExpect(status().isNotFound());
		}
	}

	@Nested
	class TestInternalError {
		@Test
		@WithMockUser
		void shouldReturn500ByTechnicalException() throws Exception {
			doThrow(TechnicalException.class).when(fileService).getFile(anyString(), anyString());

			performRequest().andExpect(status().isInternalServerError());
		}

		@Test
		@WithMockUser
		void shouldReturn500ByRuntimeException() throws Exception {
			doThrow(RuntimeException.class).when(fileService).getFile(anyString(), anyString());

			performRequest().andExpect(status().isInternalServerError());
		}
	}

	@Nested
	class TestNotAcceptable {
		@BeforeEach
		void init() {
			when(fileService.getFile(anyString(), any())).thenReturn(OzgFileTestFactory.create());
			doThrow(VirusFoundException.class).when(fileService).getFile(anyString(), any());
		}

		@Test
		@WithMockUser
		void shouldReturn406ByVirusFoundException() throws Exception {
			performRequest().andExpect(status().isNotAcceptable());
		}
	}

	ResultActions performRequest() throws Exception {
		return mockMvc.perform(
		  get(PATH + NachrichtTestFactory.NACHRICHT_EVENT_ID + "/" + FILE_ID)
			.with(csrf().asHeader())
			.accept(MediaType.APPLICATION_OCTET_STREAM_VALUE));
	}

	@Nested
	class TestFileUpload {
		private final CompletableFuture<String> fileIdFuture = CompletableFuture.completedFuture(FILE_ID);

		@BeforeEach
		void init() {
			when(fileService.upload(anyString(), anyString(), any())).thenReturn(fileIdFuture);
		}

		@Test
		@WithMockUser
		void shouldReturnCreatedStatus() throws Exception {
			var result = callEndpoint(TEST_FILE);

			mockMvc.perform(asyncDispatch(result)).andExpect(status().isCreated());
		}

		@Test
		@WithMockUser
		void shouldReturnLocation() throws Exception {
			var result = callEndpoint(TEST_FILE);

			mockMvc.perform(asyncDispatch(result))
			  .andExpect(header().string("Location", containsString(PATH_PATTERN.formatted(VORGANG_ID, FILE_ID))));
		}

		@Test
		@WithMockUser
		void shouldReturnNotAcceptableBecauseOfFileType() throws Exception {
			var result = callEndpoint(TEST_INVALID_FILE);

			mockMvc.perform(asyncDispatch(result))
			  .andExpect(status().is(HttpStatus.NOT_ACCEPTABLE.value()));
		}

		@Test
		@WithMockUser
		void shouldReturnNotAcceptableBecauseOfFileSize() throws Exception {
			var result = callEndpoint(TEST_TOO_LARGE_FILE);

			mockMvc.perform(asyncDispatch(result))
			  .andExpect(status().is(HttpStatus.NOT_ACCEPTABLE.value()));
		}

		private MvcResult callEndpoint(MockMultipartFile file) throws Exception {
			return mockMvc.perform(multipart(PATH_PATTERN.formatted(VORGANG_ID, NACHRICHT_EVENT_ID))
				.file(file)
				.with(csrf())
				.contentType(MediaType.MULTIPART_FORM_DATA_VALUE))
			  .andExpect(request().asyncStarted())
			  .andReturn();
		}
	}
}
