/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.attachments;

import static de.ozgcloud.antragsraum.attachments.OzgFileTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;

import com.google.common.collect.ImmutableList;
import com.google.protobuf.ByteString;

import de.ozgcloud.antragsraum.callcontext.ContextService;
import de.ozgcloud.antragsraum.common.ChannelPropertiesFactory;
import de.ozgcloud.vorgang.grpc.binaryFile.BinaryFileServiceGrpc;
import de.ozgcloud.vorgang.grpc.command.GrpcCallContext;
import net.devh.boot.grpc.client.config.GrpcChannelsProperties;
import net.devh.boot.grpc.client.interceptor.GlobalClientInterceptorRegistry;

@ExtendWith(MockitoExtension.class)
class FileGrpcClientTest {
	@Spy
	@InjectMocks
	private FileGrpcClient fileGrpcClient;

	@Mock
	private GlobalClientInterceptorRegistry registry;

	@Mock
	private ChannelPropertiesFactory channelPropertiesFactory;

	@Mock
	private ContextService contextService;

	@Nested
	class TestCreatingChannel {
		@BeforeEach
		void init() {
			when(channelPropertiesFactory.getGrpcChannelsProperties(anyString())).thenReturn(new GrpcChannelsProperties());
		}

		@Test
		void shouldGetChannelFactory() {
			var channelFactory = fileGrpcClient.getChannelFactory(CHANNEL_ADDRESS);
			assertThat(channelFactory).isNotNull();
		}

		@Test
		void shouldGetChannel() {
			when(registry.getClientInterceptors()).thenReturn(ImmutableList.of());

			var channel = fileGrpcClient.getChannelFactory(CHANNEL_ADDRESS).createChannel(CHANNEL_ADDRESS);

			assertThat(channel).isNotNull();
		}
	}

	@Nested
	class TestDownloadObserver {
		@Test
		void shouldCreateDownloadObserver() {
			var observer = fileGrpcClient.createDownloadFileObserver(CompletableFuture.completedFuture(Boolean.TRUE), new ByteArrayOutputStream());

			assertThat(observer).isNotNull();
		}
	}

	@Nested
	class TestUploadingFile {
		@Mock
		private BinaryFileServiceGrpc.BinaryFileServiceStub asyncStub;

		private final OzgUploadFile file = OzgUploadFileTestFactory.create();

		CompletableFuture<String> fileIdFuture = new CompletableFuture<>();

		@BeforeEach
		void init() {
			when(contextService.createCallContext()).thenReturn(GrpcCallContext.newBuilder().build());
			when(registry.getClientInterceptors()).thenReturn(ImmutableList.of());
			when(channelPropertiesFactory.getGrpcChannelsProperties(anyString())).thenReturn(
			  new GrpcChannelsProperties());
		}

		@Test
		void shouldCallGetChannel() {
			fileGrpcClient.uploadFile(file, CHANNEL_ADDRESS, fileIdFuture);

			verify(fileGrpcClient).getChannelFactory(anyString());
		}

		@Test
		void shouldCallBuildCallStreamObserver() {
			fileGrpcClient.uploadFile(file, CHANNEL_ADDRESS, fileIdFuture);

			verify(fileGrpcClient).createUploadFileObserver(any(), any());
		}
	}

	@Nested
	class TestBuildChunkRequest {
		@Test
		void shouldContainContent() {
			var chunkRequest = fileGrpcClient.buildChunkRequest(DATA);

			assertThat(chunkRequest.getFileContent()).isEqualTo(ByteString.copyFrom(DATA));
		}
	}

	@Nested
	class TestDownloadingFiles {
		@Mock
		private FileDownloadStreamObserver responseObserver;

		@Mock
		private BinaryFileServiceGrpc.BinaryFileServiceStub asyncStub;

		private final ByteArrayOutputStream output = new ByteArrayOutputStream();

		@BeforeEach
		void init() {
			doReturn(responseObserver).when(fileGrpcClient).createDownloadFileObserver(any(), any(OutputStream.class));
			doNothing().when(fileGrpcClient).waitUntilFutureToComplete(any());
			when(registry.getClientInterceptors()).thenReturn(ImmutableList.of());
			when(channelPropertiesFactory.getGrpcChannelsProperties(anyString())).thenReturn(
			  new GrpcChannelsProperties());
		}

		@Test
		void shouldCallCreateDownloadFileObserver() {
			fileGrpcClient.downloadFileContent(FILE_ID, output, CHANNEL_ADDRESS);

			verify(fileGrpcClient).createDownloadFileObserver(any(), any(OutputStream.class));
		}
	}

	@Nested
	class TestWaitUntilFutureToComplete {
		@Mock
		private CompletableFuture<Boolean> streamFuture;

		@Test
		void shouldNotThrowException() {
			assertDoesNotThrow(() -> fileGrpcClient.waitUntilFutureToComplete(streamFuture));
		}

		@ParameterizedTest
		@ValueSource(classes = { InterruptedException.class, ExecutionException.class, TimeoutException.class })
		void shouldRethrowAsRuntimeException(Class<Exception> exception)
		  throws InterruptedException, ExecutionException, TimeoutException {
			doThrow(exception).when(streamFuture).get(anyLong(), any(TimeUnit.class));

			assertThrows(RuntimeException.class, () -> fileGrpcClient.waitUntilFutureToComplete(streamFuture));
		}
	}
}