/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.attachments;

import static de.ozgcloud.antragsraum.attachments.OzgFileTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Set;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.web.multipart.MultipartFile;

import de.ozgcloud.antragsraum.common.TechnicalException;
import de.ozgcloud.antragsraum.common.VirusFoundException;
import de.ozgcloud.antragsraum.events.NachrichtEventService;
import de.ozgcloud.antragsraum.events.NachrichtEventTestFactory;

@ExtendWith(MockitoExtension.class)
class FileServiceTest {
	@Spy
	@InjectMocks
	private FileService fileService;

	@Mock
	private FileRemoteService remoteService;

	@Mock
	private VirusScannerClient virusScannerClient;

	@Mock
	private NachrichtEventService nachrichtEventService;

	@Nested
	class TestFileUpload {
		@Mock
		private MultipartFile multipartFile;

		@BeforeEach
		void init() throws IOException {
			when(remoteService.uploadFile(any(OzgUploadFile.class), anyString())).thenReturn(
			  CompletableFuture.completedFuture(FILE_ID));
			when(multipartFile.getInputStream()).thenReturn(new ByteArrayInputStream(DATA));
			when(nachrichtEventService.getNachrichtEventById(NACHRICHT_EVENT_ID)).thenReturn(NachrichtEventTestFactory.createNachrichtEvent());
		}

		@Test
		void shouldCallRemoteService() {
			fileService.upload(VORGANG_ID, NACHRICHT_EVENT_ID, multipartFile);

			verify(remoteService, times(1)).uploadFile(any(OzgUploadFile.class), anyString());
		}

		@Test
		void shouldGetFileId() {
			var res = fileService.upload(VORGANG_ID, NACHRICHT_EVENT_ID, multipartFile);

			try {
				assertThat(res.get(5, TimeUnit.SECONDS)).isEqualTo(FILE_ID);
			} catch (InterruptedException | ExecutionException | TimeoutException e) {
				fail("Exception happened! " + e.getMessage());
			}
		}
	}

	@Nested
	class TestExceptionWhenSendingFile {
		@Mock
		private MultipartFile multipartFile;

		@Test
		void shouldThrowTechnicalExceptionOnUpload() throws IOException {
			doThrow(IOException.class).when(multipartFile).getInputStream();

			assertThatExceptionOfType(TechnicalException.class).isThrownBy(
				() -> fileService.upload(VORGANG_ID, NACHRICHT_EVENT_ID, multipartFile))
			  .withMessage("TechnicalException happened! Message: null");
		}
	}

	@Nested
	class TestLoadingFile {
		@BeforeEach
		void init() {
			when(remoteService.getFile(anyString(), anyString())).thenReturn(OzgFileTestFactory.create());
			when(nachrichtEventService.getNachrichtEventById(NACHRICHT_EVENT_ID)).thenReturn(NachrichtEventTestFactory.createNachrichtEvent());
		}

		@Test
		void shouldGetFile() {
			var file = fileService.getFile(FILE_ID, NACHRICHT_EVENT_ID);

			assertThat(file).isNotNull();
		}
	}

	@Nested
	class TestDownloadingFileContent {
		@BeforeEach
		void init() {
			when(nachrichtEventService.getNachrichtEventById(NACHRICHT_EVENT_ID)).thenReturn(NachrichtEventTestFactory.createNachrichtEvent());
		}

		@Test
		void shouldCallRemoteService() {
			fileService.downloadFileContent(FILE_ID, new ByteArrayOutputStream(), NACHRICHT_EVENT_ID);

			verify(remoteService).downloadFileContent(anyString(), any(OutputStream.class), anyString());
		}
	}

	@Nested
	class TestVirusScanning {
		@Mock
		private MultipartFile multipartFile;

		@Test
		void shouldThrowVirusFoundExceptionWhenUploadingFile() {
			when(virusScannerClient.scan(multipartFile)).thenReturn(Set.of("VirusA", "VirusB"));

			assertThatExceptionOfType(VirusFoundException.class).isThrownBy(
			  () -> fileService.createUploadFile(VORGANG_ID, multipartFile));
		}
	}
}