/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.attachments;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatExceptionOfType;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.when;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.test.util.ReflectionTestUtils;
import org.springframework.web.client.RestTemplate;

@ExtendWith(MockitoExtension.class)
public class VirusScannerClientTest {
    private static final String CLAMAV_SCAN_URL = "TestClamAVScanUrl";
    private static final String FILE_NAME = "MockFileName.pdf";

    @Spy
    @InjectMocks
    private VirusScannerClient virusScannerClient;

    @Mock
    private RestTemplate restTemplate;

    @BeforeEach
    void setup() {
        ReflectionTestUtils.setField(virusScannerClient, "clamAVScanUrl", CLAMAV_SCAN_URL);
    }

    @Nested
    class TestVirusScanNotSuccessful {

        @Test
        void shouldThrowRuntimeExceptionOnClamAVCall() {
            var file = new MockMultipartFile(FILE_NAME, new byte[0]);
            var status = HttpStatus.INTERNAL_SERVER_ERROR;
            var response = new ResponseEntity<>(VirusScannerClientTestFactory.createVirusScanResponse(), status);

            when(restTemplate.postForEntity(eq(CLAMAV_SCAN_URL), any(HttpEntity.class),
                eq(VirusScannerClient.VirusScanResponse.class))).thenReturn(response);

            assertThatExceptionOfType(RuntimeException.class).isThrownBy(() -> virusScannerClient.scan(file));
        }

        @Test
        void shouldThrowRuntimeExceptionOnScan() {
            var file = new MockMultipartFile(FILE_NAME, new byte[0]);
            var status = HttpStatus.OK;
            var response =
                new ResponseEntity<>(VirusScannerClientTestFactory.createVirusScanResponse(false, false), status);

            when(restTemplate.postForEntity(eq(CLAMAV_SCAN_URL), any(HttpEntity.class),
                eq(VirusScannerClient.VirusScanResponse.class))).thenReturn(response);

            assertThatExceptionOfType(RuntimeException.class).isThrownBy(() -> virusScannerClient.scan(file));
        }
    }

    @Nested
    class TestVirusScanSuccessful {

        @Test
        void shouldHaveTwoVirusesInResult() {
            var file = new MockMultipartFile(FILE_NAME, new byte[0]);
            var status = HttpStatus.OK;
            var response =
                new ResponseEntity<>(VirusScannerClientTestFactory.createVirusScanResponse(true, true), status);

            when(restTemplate.postForEntity(eq(CLAMAV_SCAN_URL), any(HttpEntity.class),
                eq(VirusScannerClient.VirusScanResponse.class))).thenReturn(response);

            var result = virusScannerClient.scan(file);
            assertThat(result).hasSize(2);
        }

        @Test
        void shouldHaveEmptyResult() {
            var file = new MockMultipartFile(FILE_NAME, new byte[0]);
            var status = HttpStatus.OK;
            var response = new ResponseEntity<>(VirusScannerClientTestFactory.createVirusScanResponse(), status);

            when(restTemplate.postForEntity(eq(CLAMAV_SCAN_URL), any(HttpEntity.class),
                eq(VirusScannerClient.VirusScanResponse.class))).thenReturn(response);

            var result = virusScannerClient.scan(file);
            assertThat(result).isEmpty();
        }
    }
}
