/*
 * Copyright (c) 2023-2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.events;

import static org.assertj.core.api.Assertions.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;

import de.ozgcloud.info.GrpcInformationByIdRequest;
import de.ozgcloud.info.GrpcInformationByIdResponse;
import de.ozgcloud.info.GrpcInformationNachricht;
import de.ozgcloud.info.GrpcInformationRequest;
import de.ozgcloud.info.GrpcInformationResponse;
import de.ozgcloud.info.InformationServiceGrpc;
import io.grpc.Status;
import io.grpc.StatusRuntimeException;

@ExtendWith(MockitoExtension.class)
class NachrichtEventRemoteServiceTest {
	@Spy
	@InjectMocks
	private NachrichtEventRemoteService remoteService;
	@Mock
	private InformationServiceGrpc.InformationServiceBlockingStub informationServiceBlockingClient;

	@Nested
	class TestInformationAvailable {
		private final List<GrpcInformationNachricht> nachrichten = List.of(NachrichtEventTestFactory.createGrpcInformationNachricht());

		@BeforeEach
		void init() {
			when(informationServiceBlockingClient.getInformation(any(GrpcInformationRequest.class)))
			  .thenReturn(GrpcInformationResponse.newBuilder().addAllNachrichten(nachrichten).build());
		}

		@Test
		void shouldLoadNachrichtEvents() {
			var events = remoteService.getNachrichtEventsOfPostfach(NachrichtEventTestFactory.POSTFACH_ID);

			assertThat(events).hasSize(1);
		}

		@Test
		void shouldHavePostfachId() {
			var events = remoteService.getNachrichtEventsOfPostfach(NachrichtEventTestFactory.POSTFACH_ID);

			assertThat(events.getFirst().postfachId()).isEqualTo(NachrichtEventTestFactory.POSTFACH_ID);
		}
	}

	@Nested
	class TestNoInformationAvailable {
		private final List<GrpcInformationNachricht> nachrichten = List.of();

		@BeforeEach
		void init() {
			when(informationServiceBlockingClient.getInformation(any(GrpcInformationRequest.class)))
			  .thenReturn(GrpcInformationResponse.newBuilder().addAllNachrichten(nachrichten).build());
		}

		@Test
		void shouldLoadEmptyNachrichtEvents() {
			var events = remoteService.getNachrichtEventsOfPostfach(NachrichtEventTestFactory.POSTFACH_ID);

			assertThat(events).hasSize(0);
		}
	}

	@Nested
	class TestInformationByIdAvailable {
		private final GrpcInformationNachricht nachricht = NachrichtEventTestFactory.createGrpcInformationNachricht();

		@BeforeEach
		void init() {
			when(informationServiceBlockingClient.getInformationById(any(GrpcInformationByIdRequest.class)))
			  .thenReturn(GrpcInformationByIdResponse.newBuilder().setNachricht(nachricht).build());
		}

		@Test
		void shouldLoadNachrichtEventById() {
			var event = remoteService.getNachrichtEventById(NachrichtEventTestFactory.ID);

			assertThat(event).isInstanceOf(NachrichtEvent.class);
		}

		@Test
		void shouldHavePostfachId() {
			var event = remoteService.getNachrichtEventById(NachrichtEventTestFactory.ID);

			assertThat(event.postfachId()).isEqualTo(NachrichtEventTestFactory.POSTFACH_ID);
		}
	}

	@Nested
	class TestNoInformationByIdAvailable {
		@BeforeEach
		void init() {
			when(informationServiceBlockingClient.getInformationById(any(GrpcInformationByIdRequest.class)))
			  .thenThrow(new StatusRuntimeException(Status.INVALID_ARGUMENT));
		}

		@Test
		void shouldLoadEmptyNachrichtEvents() {
			assertThrows(StatusRuntimeException.class, () -> remoteService.getNachrichtEventById(InformationGrpcServiceStub.ID));
		}
	}
}