/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.nachricht;

import static de.ozgcloud.antragsraum.nachricht.GrpcRueckfrageTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import de.ozgcloud.antragsraum.attachments.FileService;
import de.ozgcloud.antragsraum.attachments.OzgFile;
import de.ozgcloud.antragsraum.attachments.OzgFileTestFactory;
import de.ozgcloud.nachrichten.antragraum.GrpcRueckfrage;
import de.ozgcloud.nachrichten.antragraum.GrpcRueckfrageAnswer;

@ExtendWith(MockitoExtension.class)
class NachrichtMapperTest {
	@Mock
	private FileService fileService;

	@InjectMocks
	private NachrichtMapper mapper;

	@Nested
	class TestFromGrpcRueckfrage {
		private final GrpcRueckfrage rueckfrage = GrpcRueckfrageTestFactory.createRueckfrage();
		private final GrpcRueckfrage answeredRueckfrage = GrpcRueckfrageTestFactory.createRueckfrageWithReply();

		@BeforeEach
		void setup() {
			when(fileService.getFile(anyString(), anyString())).thenReturn(OzgFileTestFactory.create());
		}

		@Test
		void shouldMapNachrichtId() {
			var nachricht = mapper.fromGrpcRueckfrage(GrpcRueckfrageTestFactory.createRueckfrageBuilder().setId(NachrichtTestFactory.ID).build(),
			  NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.id()).isEqualTo(NachrichtTestFactory.ID);
		}

		@Test
		void shouldMapNachrichtEventId() {
			var nachricht = mapper.fromGrpcRueckfrage(GrpcRueckfrageTestFactory.createRueckfrageBuilder().setId(NachrichtTestFactory.ID).build(),
			  NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.nachrichtEventId()).isEqualTo(NachrichtTestFactory.NACHRICHT_EVENT_ID);
		}

		@Test
		void shouldMapVorgangId() {
			var nachricht = mapper.fromGrpcRueckfrage(rueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.vorgangId()).isEqualTo(GrpcRueckfrageTestFactory.VORGANG_ID);
		}

		@Test
		void shouldMapVorgangName() {
			var nachricht = mapper.fromGrpcRueckfrage(rueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.title()).isEqualTo(NachrichtTestFactory.VORGANG_TITLE);
		}

		@Test
		void shouldMapNachrichtText() {
			var nachricht = mapper.fromGrpcRueckfrage(rueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.message()).isEqualTo(GrpcRueckfrageTestFactory.TEXT);
		}

		@Test
		void shouldMapAnsweredAt() {
			var nachricht = mapper.fromGrpcRueckfrage(answeredRueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.answeredAt()).isEqualTo(GrpcRueckfrageTestFactory.ANSWERED_AT_DATE);
		}

		@Test
		void shouldMapSendAt() {
			var nachricht = mapper.fromGrpcRueckfrage(rueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.sendAt()).isEqualTo(GrpcRueckfrageTestFactory.SEND_AT_DATE);
		}

		@Test
		void shouldMapInvalidAnsweredAt() {
			var nachricht = mapper.fromGrpcRueckfrage(GrpcRueckfrageTestFactory.createRueckfrageBuilder().setAnsweredAt("huu").build(),
			  NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.answeredAt()).isZero();
		}

		@Test
		void shouldMapAttachments() {
			var nachricht = mapper.fromGrpcRueckfrage(GrpcRueckfrageTestFactory.createRueckfrage(), NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.attachments()).hasSize(2);
		}

		@Test
		void shouldMapEmptyAttachments() {
			when(fileService.getFile(anyString(), anyString())).thenReturn(null);

			var nachricht = mapper.fromGrpcRueckfrage(GrpcRueckfrageTestFactory.createRueckfrage(), NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.attachments()).isEmpty();
		}

		@Test
		void shouldMapAccessible() {
			var nachricht = mapper.fromGrpcRueckfrage(answeredRueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.accessible()).isEqualTo(NachrichtTestFactory.ACCESSIBLE);
		}

		@Test
		void shouldMapTrustLevel() {
			var nachricht = mapper.fromGrpcRueckfrage(answeredRueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.trustLevel()).isEqualTo(NachrichtTestFactory.TRUST_LEVEL);
		}

		@Test
		void shouldNotHaveAnswersWhenUnanswered() {
			var nachricht = mapper.fromGrpcRueckfrage(GrpcRueckfrageTestFactory.createRueckfrage(), NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.replyNachrichten()).isEmpty();
		}

		@Test
		void shouldMapAnswerNachrichtText() {
			var nachricht = mapper.fromGrpcRueckfrage(answeredRueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.replyNachrichten().getFirst().message()).isEqualTo(GrpcRueckfrageTestFactory.ANSWER_TEXT);
		}

		@Test
		void shouldMapAnswerAttachments() {
			var nachricht = mapper.fromGrpcRueckfrage(answeredRueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.replyNachrichten().getFirst().attachments()).hasSize(1);
		}

		@Test
		void shouldMapStatus() {
			var nachricht = mapper.fromGrpcRueckfrage(rueckfrage, NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.status()).isEqualTo(NachrichtTestFactory.STATUS);
		}
	}

	@Nested
	class TestGetAnsweredAt {
		@Test
		void shouldGetAnsweredAtWhenAnsweredAtIsSet() {
			GrpcRueckfrage answeredRueckfrage = GrpcRueckfrageTestFactory.createRueckfrageWithReply();

			var date = mapper.getAnsweredAt(answeredRueckfrage);

			assertThat(date).isEqualTo(mapper.toMillisecondsTimestamp(answeredRueckfrage.getAnsweredAt()));
		}

		@Test
		void shouldGetAnsweredAtWhenAnsweredAtIsNotSetButAnswersHaveSendDat() {
			GrpcRueckfrage answeredRueckfrage = GrpcRueckfrageTestFactory.createRueckfrageBuilder()
			  .setAnsweredAt("")
			  .clearAnswers()
			  .addAnswers(GrpcRueckfrageAnswer.newBuilder().setSentAt(SEND_AT).build())
			  .addAnswers(GrpcRueckfrageAnswer.newBuilder().setSentAt(ANSWERED_AT).build())
			  .build();

			var date = mapper.getAnsweredAt(answeredRueckfrage);

			assertThat(date).isEqualTo(mapper.toMillisecondsTimestamp(ANSWERED_AT));
		}

		@Test
		void shouldGetAnsweredAtWhenAnsweredAtIsNotSetAtAll() {
			GrpcRueckfrage answeredRueckfrage = GrpcRueckfrageTestFactory.createRueckfrageBuilder()
			  .setAnsweredAt("")
			  .clearAnswers()
			  .addAnswers(GrpcRueckfrageAnswer.newBuilder().build())
			  .addAnswers(GrpcRueckfrageAnswer.newBuilder().build())
			  .build();

			var date = mapper.getAnsweredAt(answeredRueckfrage);

			assertThat(date).isEqualTo(0);
		}

		@Test
		void shouldGetAnsweredWhenSecondSendAtIsNull() {
			GrpcRueckfrage answeredRueckfrage = GrpcRueckfrageTestFactory.createRueckfrageBuilder()
			  .setAnsweredAt("")
			  .clearAnswers()
			  .addAnswers(GrpcRueckfrageAnswer.newBuilder().setSentAt(SEND_AT_LONG).build())
			  .addAnswers(GrpcRueckfrageAnswer.newBuilder().build())
			  .build();

			var date = mapper.getAnsweredAt(answeredRueckfrage);

			assertThat(date).isEqualTo(mapper.toMillisecondsTimestamp(SEND_AT));
		}

		@Test
		void shouldGetAnsweredWhenFirstSendAtIsNull() {
			GrpcRueckfrage answeredRueckfrage = GrpcRueckfrageTestFactory.createRueckfrageBuilder()
			  .setAnsweredAt("")
			  .clearAnswers()
			  .addAnswers(GrpcRueckfrageAnswer.newBuilder().build())
			  .addAnswers(GrpcRueckfrageAnswer.newBuilder().setSentAt(SEND_AT).build())
			  .build();

			var date = mapper.getAnsweredAt(answeredRueckfrage);

			assertThat(date).isEqualTo(mapper.toMillisecondsTimestamp(SEND_AT));
		}
	}

	@Nested
	class TestToGrpcRueckfrageAnswer {
		@Test
		void shouldMapNachrichtText() {
			var nachricht = mapper.toGrpcRueckfrageAnswer(ReplyNachrichtTestFactory.create());

			assertThat(nachricht.getAnswerText()).isEqualTo(ReplyNachrichtTestFactory.TEXT);
		}

		@Test
		void shouldMapEmptyAttachments() {
			var nachricht = mapper.toGrpcRueckfrageAnswer(ReplyNachrichtTestFactory.create());

			assertThat(nachricht.getAttachmentFileIdList()).isEmpty();
		}

		@Test
		void shouldMapAttachments() {
			var nachricht = mapper.toGrpcRueckfrageAnswer(
			  ReplyNachrichtTestFactory.createBuilder()
				.attachments(List.of(OzgFile.builder().id(UUID.randomUUID().toString()).build())).build());

			assertThat(nachricht.getAttachmentFileIdList()).hasSize(1);
		}

		@Test
		void shouldNotMapEmptyAttachments() {
			var nachricht = mapper.fromGrpcRueckfrage(
			  GrpcRueckfrageTestFactory.createRueckfrageBuilder().clearAttachmentFileId().addAttachmentFileId("").build(),
			  NachrichtTestFactory.NACHRICHT_EVENT_ID);

			assertThat(nachricht.attachments()).isEmpty();
		}
	}
}