/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.antragsraum.nachricht;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;

import com.google.common.collect.ImmutableList;

import de.ozgcloud.antragsraum.common.ChannelPropertiesFactory;
import de.ozgcloud.nachrichten.antragraum.AntragraumServiceGrpc;
import de.ozgcloud.nachrichten.antragraum.GrpcFindRueckfragenRequest;
import de.ozgcloud.nachrichten.antragraum.GrpcFindRueckfragenResponse;
import de.ozgcloud.nachrichten.antragraum.GrpcGetRueckfrageRequest;
import de.ozgcloud.nachrichten.antragraum.GrpcGetRueckfrageResponse;
import de.ozgcloud.nachrichten.antragraum.GrpcSendRueckfrageAnswerRequest;
import de.ozgcloud.nachrichten.antragraum.GrpcSendRueckfrageAnswerResponse;
import io.grpc.Channel;
import net.devh.boot.grpc.client.channelfactory.GrpcChannelFactory;
import net.devh.boot.grpc.client.config.GrpcChannelsProperties;
import net.devh.boot.grpc.client.interceptor.GlobalClientInterceptorRegistry;

@ExtendWith(MockitoExtension.class)
class NachrichtenGrpcClientTest {
	private final static String CHANNEL_ADDRESS = "static://localhost:9090";

	@Spy
	@InjectMocks
	private NachrichtenGrpcClient grpcClient;

	@Mock
	private GlobalClientInterceptorRegistry registry;

	@Mock
	private ChannelPropertiesFactory channelPropertiesFactory;

	@Nested
	class TestCreatingChannel {
		@BeforeEach
		void init() {
			when(channelPropertiesFactory.getGrpcChannelsProperties(anyString())).thenReturn(new GrpcChannelsProperties());
		}

		@Test
		void shouldGetChannelFactory() {
			var channelFactory = grpcClient.getChannelFactory(CHANNEL_ADDRESS);

			assertThat(channelFactory).isNotNull();
		}

		@Test
		void shouldGetChannel() {
			when(registry.getClientInterceptors()).thenReturn(ImmutableList.of());

			var channel = grpcClient.getChannelFactory(CHANNEL_ADDRESS).createChannel(CHANNEL_ADDRESS);

			assertThat(channel).isNotNull();
		}
	}

	@Nested
	class TestSendingRequests {
		@Mock
		private AntragraumServiceGrpc.AntragraumServiceBlockingStub antragraumServiceBlockingStub;

		@Nested
		class TestRueckfrageAnswer {
			@Mock
			private GrpcSendRueckfrageAnswerRequest request;

			@Test
			void shouldGetAnswerResponse() {
				var channelFactory = mock(GrpcChannelFactory.class);
				var channel = mock(Channel.class);
				when(channelFactory.createChannel(anyString())).thenReturn(channel);
				doReturn(channelFactory).when(grpcClient).getChannelFactory(anyString());
				doReturn(GrpcSendRueckfrageAnswerResponse.getDefaultInstance()).when(grpcClient)
				  .getAnswerResponse(any(GrpcSendRueckfrageAnswerRequest.class), any());

				grpcClient.answerRueckfrage(request, CHANNEL_ADDRESS);

				verify(grpcClient).getAnswerResponse(any(GrpcSendRueckfrageAnswerRequest.class), any());
			}

			@Test
			void shouldSendRueckfrageAnswer() {
				grpcClient.getAnswerResponse(request, antragraumServiceBlockingStub);

				verify(antragraumServiceBlockingStub).sendRueckfrageAnswer(any(GrpcSendRueckfrageAnswerRequest.class));
			}
		}

		@Nested
		class TestFindRueckfragen {
			@Mock
			private GrpcFindRueckfragenRequest request;

			@Test
			void shouldCallClientFindRueckfragen() {
				var channelFactory = mock(GrpcChannelFactory.class);
				var channel = mock(Channel.class);
				when(channelFactory.createChannel(anyString())).thenReturn(channel);
				doReturn(channelFactory).when(grpcClient).getChannelFactory(anyString());
				doReturn(GrpcFindRueckfragenResponse.getDefaultInstance()).when(grpcClient)
				  .getGrpcFindRueckfragenResponse(any(GrpcFindRueckfragenRequest.class), any());

				grpcClient.getFindRueckfragen(request, CHANNEL_ADDRESS);

				verify(grpcClient).getGrpcFindRueckfragenResponse(any(GrpcFindRueckfragenRequest.class), any());
			}

			@Test
			void shouldCallRemoteServiceFindRueckfragen() {
				grpcClient.getGrpcFindRueckfragenResponse(request, antragraumServiceBlockingStub);

				verify(antragraumServiceBlockingStub).findRueckfragen(any(GrpcFindRueckfragenRequest.class));
			}
		}

		@Nested
		class TestGetRueckfrage {
			@Mock
			private GrpcGetRueckfrageRequest request;

			@Test
			void shouldCallClientGetRueckfrage() {
				var channelFactory = mock(GrpcChannelFactory.class);
				var channel = mock(Channel.class);
				when(channelFactory.createChannel(anyString())).thenReturn(channel);
				doReturn(channelFactory).when(grpcClient).getChannelFactory(anyString());
				doReturn(GrpcGetRueckfrageResponse.getDefaultInstance()).when(grpcClient)
				  .getGrpcGetRueckfrageResponse(any(GrpcGetRueckfrageRequest.class), any());

				grpcClient.getGetRueckfrage(request, CHANNEL_ADDRESS);

				verify(grpcClient).getGrpcGetRueckfrageResponse(any(GrpcGetRueckfrageRequest.class), any());
			}

			@Test
			void shouldCallRemoteServiceFindRueckfragen() {
				grpcClient.getGrpcGetRueckfrageResponse(request, antragraumServiceBlockingStub);

				verify(antragraumServiceBlockingStub).getRueckfrage(any(GrpcGetRueckfrageRequest.class));
			}
		}
	}
}