/*
 * Copyright (c) 2023-2024.
 * Das Land Schleswig-Holstein vertreten durch den Ministerpräsidenten des Landes Schleswig-Holstein Staatskanzlei Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.antragsraum.nachricht;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.ArgumentCaptor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;

import de.ozgcloud.antragsraum.callcontext.ContextService;
import de.ozgcloud.antragsraum.command.CommandReference;
import de.ozgcloud.antragsraum.command.CommandTestFactory;
import de.ozgcloud.antragsraum.common.AddressNotFoundException;
import de.ozgcloud.antragsraum.events.NachrichtEvent;
import de.ozgcloud.antragsraum.events.NachrichtEventTestFactory;
import de.ozgcloud.nachrichten.antragraum.GrpcFindRueckfragenRequest;
import de.ozgcloud.nachrichten.antragraum.GrpcGetRueckfrageRequest;
import de.ozgcloud.nachrichten.antragraum.GrpcRueckfrageAnswer;
import de.ozgcloud.nachrichten.antragraum.GrpcSendRueckfrageAnswerRequest;
import io.grpc.StatusRuntimeException;

@ExtendWith(MockitoExtension.class)
public class NachrichtenRemoteServiceTest {
	public static final String ADDRESS = "test";
	public static final String POSTFACH_ID = UUID.randomUUID().toString();
	public static final String SAML_TOKEN = "saml-token";

	@Spy
	@InjectMocks
	private NachrichtenRemoteService nachrichtenRemoteService;

	@Mock
	private NachrichtenGrpcClient nachrichtenGrpcClient;

	@Mock
	private NachrichtMapper nachrichtMapper;

	@Mock
	private ContextService contextService;

	@Nested
	class TestNachrichtenAvailable {
		private final NachrichtEvent nachrichtEvent = NachrichtEventTestFactory.createNachrichtEvent();

		@BeforeEach
		void init() {
			when(contextService.getSamlResponse()).thenReturn(SAML_TOKEN);
			when(nachrichtenGrpcClient.getFindRueckfragen(any(GrpcFindRueckfragenRequest.class), anyString())).thenReturn(
			  GrpcRueckfrageTestFactory.createFindRueckfragenResponse());
		}

		@Test
		void shouldLoadNachrichtHeaders() {
			var nachrichtHeaders = nachrichtenRemoteService.findRueckfrageHeads(nachrichtEvent);

			assertThat(nachrichtHeaders).isNotEmpty();
		}

		@Test
		void shouldSetSamlToken() {
			ArgumentCaptor<GrpcFindRueckfragenRequest> requestArgumentCaptor = ArgumentCaptor.forClass(GrpcFindRueckfragenRequest.class);

			nachrichtenRemoteService.findRueckfrageHeads(nachrichtEvent);

			verify(nachrichtenGrpcClient).getFindRueckfragen(requestArgumentCaptor.capture(), anyString());
			assertThat(requestArgumentCaptor.getValue().getSamlToken()).isNotEmpty();
		}
	}

	@Nested
	class TestNoNachrichtenAvailable {
		@BeforeEach
		void init() {
			when(contextService.getSamlResponse()).thenReturn(SAML_TOKEN);
			doThrow(StatusRuntimeException.class).when(nachrichtenGrpcClient).getFindRueckfragen(any(GrpcFindRueckfragenRequest.class), anyString());
		}

		@Test
		void shouldHandleStatusRuntimeException() {
			var nachrichtHeaders = nachrichtenRemoteService.findRueckfrageHeads(NachrichtEventTestFactory.createNachrichtEvent());

			assertThat(nachrichtHeaders).isEmpty();
		}
	}

	@Nested
	class TestNoTargetAddressAvailable {
		@BeforeEach
		void init() {
			when(contextService.getSamlResponse()).thenReturn(SAML_TOKEN);
			doThrow(AddressNotFoundException.class).when(nachrichtenGrpcClient)
			  .getFindRueckfragen(any(GrpcFindRueckfragenRequest.class), anyString());
		}

		@Test
		void shouldThrowAddressNotFoundException() {
			var nachrichtEvent = NachrichtEventTestFactory.createNachrichtEvent();

			assertThatExceptionOfType(AddressNotFoundException.class).isThrownBy(() -> nachrichtenRemoteService.findRueckfrageHeads(nachrichtEvent));
		}
	}

	@Nested
	class TestGetNachricht {
		private final NachrichtEvent nachrichtEvent = NachrichtEventTestFactory.createNachrichtEvent();

		@BeforeEach
		void init() {
			when(contextService.getSamlResponse()).thenReturn(SAML_TOKEN);
			when(nachrichtMapper.fromGrpcRueckfrage(any(), anyString())).thenReturn(NachrichtTestFactory.createNachricht());
			when(nachrichtenGrpcClient.getGetRueckfrage(any(GrpcGetRueckfrageRequest.class), anyString())).thenReturn(
			  GrpcRueckfrageTestFactory.createGetRueckfrageResponse());
		}

		@Test
		void shouldLoadNachricht() {
			var nachricht = nachrichtenRemoteService.getRueckfrage(GrpcRueckfrageTestFactory.RUECKFRAGE_ID, nachrichtEvent);

			assertThat(nachricht).isNotEmpty();
		}

		@Test
		void shouldSetSamlToken() {
			ArgumentCaptor<GrpcGetRueckfrageRequest> requestArgumentCaptor = ArgumentCaptor.forClass(GrpcGetRueckfrageRequest.class);

			var nachricht = nachrichtenRemoteService.getRueckfrage(GrpcRueckfrageTestFactory.RUECKFRAGE_ID, nachrichtEvent);

			verify(nachrichtenGrpcClient).getGetRueckfrage(requestArgumentCaptor.capture(), anyString());
			assertThat(requestArgumentCaptor.getValue().getSamlToken()).isNotEmpty();
		}
	}

	@Nested
	class TestSendAnswer {
		private final ReplyNachricht reply = ReplyNachrichtTestFactory.create();

		@BeforeEach
		void init() {
			when(contextService.getSamlResponse()).thenReturn(SAML_TOKEN);
			when(nachrichtMapper.toGrpcRueckfrageAnswer(any())).thenReturn(GrpcRueckfrageAnswer.newBuilder().build());
			when(nachrichtenGrpcClient.answerRueckfrage(any(GrpcSendRueckfrageAnswerRequest.class),
			  anyString())).thenReturn(GrpcRueckfrageTestFactory.createSendRueckfrageAnswerResponse());
		}

		@Test
		void shouldSendAnswer() {
			var result = nachrichtenRemoteService.sendAnswer(reply, ADDRESS);

			assertThat(result).isInstanceOf(CommandReference.class);
		}

		@Test
		void shouldReturnCommandReference() {
			var result = nachrichtenRemoteService.sendAnswer(reply, ADDRESS);

			assertThat(result.id()).isEqualTo(CommandTestFactory.ID);
			assertThat(result.nachrichtEventId()).isEqualTo(ReplyNachrichtTestFactory.NACHRICHT_EVENT_ID);
		}

		@Test
		void shouldSetSamlResponse() {
			ArgumentCaptor<GrpcSendRueckfrageAnswerRequest> requestArgumentCaptor = ArgumentCaptor.forClass(GrpcSendRueckfrageAnswerRequest.class);

			nachrichtenRemoteService.sendAnswer(reply, ADDRESS);

			verify(nachrichtenGrpcClient).answerRueckfrage(requestArgumentCaptor.capture(), anyString());
			assertThat(requestArgumentCaptor.getValue().getSamlToken()).isNotEmpty();
		}
	}
}
